/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* ex: set tabstop=8 softtabstop=4 shiftwidth=4 expandtab: */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The original template for this code came from nsCUPSShim.h, thought
 * it is significantly different due to differences in the APIs.
 *
 * The Original Code is the Mozilla PostScript driver printer list component.
 *
 * The Initial Developer of the Original Code is
 * Kenneth Herron <kherron+mozilla@fmailbox.com>
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Norm Jacobs <Norm.Jacobs@Sun.COM>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#ifndef nsPAPIShim_h___
#define nsPAPIShim_h___

#include "nsVoidArray.h"
#include "nsString.h"
#include "prtypes.h"
#include "psSharedCore.h"


/* Various PAPI data types. We don't #include papi headers to avoid
 * requiring PAPI to be installed on the build host (and to avoid having
 * to test for PAPI in configure).
 */
typedef int papi_status_t;
typedef void *papi_attribute_t;
typedef void *papi_service_t;
typedef void *papi_printer_t;
typedef void *papi_job_t;
typedef void *papi_job_ticket_t;
typedef void *papi_filter_t;

typedef enum {
        PAPI_ENCRYPT_IF_REQUESTED,      /* Encrypt if requested (TLS upgrade) */        PAPI_ENCRYPT_NEVER,             /* Never encrypt */
        PAPI_ENCRYPT_REQUIRED,          /* Encryption required (TLS upgrade) */
        PAPI_ENCRYPT_ALWAYS             /* Always encrypt (SSL) */
} papi_encryption_t;

#define NS_PAPI_PRINTER "PAPI/"
#define NS_PAPI_PRINTER_LEN (sizeof(NS_PAPI_PRINTER) - 1)

typedef papi_status_t (PR_CALLBACK *PapiServiceCreateType)(papi_service_t *svc,
                                        char *service_name, char *user_name,
                                        char *password,
                                        int (*authCB)(papi_service_t svc),
					papi_encryption_t encryption,
                                        void *app_data);
typedef void (PR_CALLBACK *PapiServiceDestroyType)(papi_service_t handle);

typedef papi_status_t (PR_CALLBACK *PapiPrintersListType)(papi_service_t handle,
					char **requested_attrs,
                                        papi_filter_t *filter,
                                        papi_printer_t **printers);
typedef papi_status_t (PR_CALLBACK *PapiPrinterQueryType)(papi_service_t handle,
					char *name, char **requested_attrs,
                                        papi_attribute_t **job_attributes,
					papi_printer_t *printer);
typedef papi_attribute_t **(PR_CALLBACK *PapiPrinterGetAttributeListType)(
                                        papi_printer_t printer);
typedef void (PR_CALLBACK *PapiPrinterListFreeType)(papi_printer_t *printer);

typedef papi_status_t (PR_CALLBACK *PapiJobSubmitType)(papi_service_t handle,
                                        char *printer,
                                        papi_attribute_t **job_attributes,
					papi_job_ticket_t *job_ticket,
                                        char **files, papi_job_t *job);
typedef void (PR_CALLBACK *PapiJobFreeType)(papi_job_t job);

typedef papi_status_t (PR_CALLBACK *PapiAttributeListGetStringType)(
                                        papi_attribute_t **list,
                                        void **iterator, char *name,
                                        char **vptr);
typedef papi_status_t (PR_CALLBACK *PapiAttributeListAddStringType)(
                                        papi_attribute_t ***list,
					int flags, char *name, char *value);

typedef papi_status_t (PR_CALLBACK *PapiAttributeListAddIntegerType)(
                                        papi_attribute_t ***list,
					int flags, char *name, int value);
typedef papi_status_t (PR_CALLBACK *PapiAttributeListFreeType)(
                                        papi_attribute_t **list);

#define PAPI_ATTR_APPEND	0x0001  /* Add values to attr */
#define PAPI_ATTR_REPLACE	0x0002  /* Delete existing values, then add */
#define PAPI_ATTR_EXCL		0x0004  /* Fail if attr exists */

struct PRLibrary;

class nsPAPIShim {
    public:
        nsPAPIShim() : mPAPILib(nsnull) { }
        ~nsPAPIShim();

        /**
         * Initialize this object. Attempt to load the PAPI shared
         * library and find function pointers for the supported
         * functions (see below).
         * @return PR_FALSE if the shared library could not be loaded, or if
         *                  any of the functions could not be found.
         *         PR_TRUE  for successful initialization.
         */
        PRBool Init();

        /**
         * @return PR_TRUE  if the object was initialized successfully.
         *         PR_FALSE otherwise.
         */
        PRBool IsInitialized() { return nsnull != mPAPILib; }

        /* Function pointers for supported functions. These are only
         * valid after successful initialization.
         */
        PapiServiceCreateType             mPapiServiceCreate;
        PapiServiceDestroyType            mPapiServiceDestroy;
        PapiPrintersListType              mPapiPrintersList;
        PapiPrinterQueryType              mPapiPrinterQuery;
        PapiPrinterListFreeType           mPapiPrinterListFree;
        PapiJobSubmitType	          mPapiJobSubmit;
        PapiJobFreeType                   mPapiJobFree;
        PapiPrinterGetAttributeListType   mPapiPrinterGetAttributeList;
        PapiAttributeListGetStringType    mPapiAttributeListGetString;
        PapiAttributeListAddIntegerType   mPapiAttributeListAddInteger;
        PapiAttributeListAddStringType    mPapiAttributeListAddString;
        PapiAttributeListFreeType         mPapiAttributeListFree;

	void PopulatePrinterList(nsCStringArray& aList);
	int PrintFile(char *printer, char *file, int copies, char *title);

    private:
        PRLibrary *mPAPILib;
};



#endif /* nsPAPIShim_h___ */
